﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

using Platform.Logging.FileLogger;

namespace Platform.Logging.HttpLogger
{
    [ProviderAlias("Http")]
    public class PlatformHttpLoggerProvider : PlatformBatchLoggerProvider<PlatformHttpLoggerOptions>
    {
        public PlatformHttpLoggerProvider( IConfigurationSection configuration )
            : base( configuration )
        {
            _ProcessId = Process.GetCurrentProcess().Id;
        }

        protected override string SectionName => "Http";

        private string _Url;
        private string _ServerName;
        private int _ProcessId;

        internal override PlatformBatchLogger<PlatformHttpLoggerOptions> CreateLoggerImplementation( string categoryName )
        {
            return new PlatformHttpLogger(this, categoryName);
        }

        protected override void OnConfigurationChanged()
        {
            // call base class
            base.OnConfigurationChanged();

            // store settings
            _Url = ProviderOptions.Url;
            _ServerName = ProviderOptions.ServerName ?? Environment.MachineName;
        }

        protected async override Task WriteMessagesAsync( IEnumerable<PlatformBatchLoggerMessage> messages, CancellationToken cancellationToken)
        {
            try
            {
                // get content
                var content = messages.Select(message => new
                {
                    Date = message.Timestamp,
                    Servername = _ServerName,
                    ProcessId = _ProcessId.ToString(),
                    ThreadId = message.ThreadId.ToString(),
                    Category = message.Category,
                    Level = message.LogLevel,
                    EventId = message.EventId.Id.ToString(),
                    EventName = message.EventId.Name,
                    ExceptionText = message.Exception?.ToString(),
                    MessageText = message.Message
                }).ToList();

                // encode to JSON
                var json = Newtonsoft.Json.JsonConvert.SerializeObject(content);

                // create client
                HttpClient client = new HttpClient();
                var response = await client.PostAsync(_Url, new StringContent(json, Encoding.UTF8, "application/json"));
                response.EnsureSuccessStatusCode();
            }
            catch ( Exception ex )
            {
                SimpleFileLogger.WriteLine(Configuration, "{0} Logger Failed to write messages: {1}", SectionName, ex.Message);
            }
        }
    }
}
